from __future__ import annotations
from board import Piece, Board, Move


# Находим наилучший из возможных результатов для первого игрока
def minimax(board: Board, maximizing: bool, original_player: Piece, max_depth: int = 8) -> float:
    # Базовый случай - достигнута финальная позиция
    # или максимальная глубина поиска
    if board.is_win or board.is_draw or max_depth == 0:
        return board.evaluate(original_player)

    # Рекурсивный случай - максимизируйте свою выгоду
    # или минимизируйте выгоду противника
    if maximizing:
        best_eval: float = float("-inf")  # произвольно низкая начальная точка
        for move in board.legal_moves:
            result: float = minimax(board.move(move), False, original_player, max_depth - 1)
            best_eval = max(result, best_eval)  # ищем переход с самой высокой оценкой
        return best_eval
    else:  # минимизация
        worst_eval: float = float("inf")
        for move in board.legal_moves:
            result = minimax(board.move(move), True, original_player, max_depth - 1)
            worst_eval = min(result, worst_eval)  # ищем переход с самой низкой оценкой
        return worst_eval


# Найти наилучший возможный ход из текущей позиции,
# просматривая тax_depth ходов вперед
def find_best_move(board: Board, max_depth: int = 8) -> Move:
    best_eval: float = float("-inf")
    best_move: Move = Move(-1)
    for move in board.legal_moves:
        result: float = minimax(board.move(move), False, board.turn, max_depth)
        if result > best_eval:
            best_eval = result
            best_move = move
    return best_move
